function c = gfmul(a, b, p)
%GFMUL  Element-by-element multiply in GF(P) or GF(P^M).
%       C = BFMUL(A, B) multiplies A with B in GF(2) element-by-element. A and
%       B are scalar or vectors in GF(2).
%
%       C = BFMUL(A, B, P) multiplies A with B in GF(P) element-by-element
%       when P is a prime number. In this case, A and B are scalars or vectors
%       in GF(P). When P is a matrix that contains the list of all tuple
%       elements in GF(p^M), this function multiplies A with B in GF(P^M). The
%       output C is alpha^C = alpha^A * alpha^B in GF(Q^M). In this case A and
%       B are scalars or vectors in GF(p^M).  You can generate the tuple of all
%       elements in GF(Q^M) by P = GFTUPLE([-1:Q^M-2]', M, Q). 
%
%       The element in GF(p^M) is represented by power form.
%       [-Inf, 0, 1, 2,...] represents [0, 1, alpha, alpha^2,...] in GF(p^m).
%
%       See also GFADD, GFDIV, GFTUPLE.

%       Wes Wang 6/8/94, 10/7/95
%       Copyright (c) 1995-96 by The MathWorks, Inc.
%       $Revision: 1.1 $  $Date: 1996/04/01 17:59:01 $

%input validation
a = a(:)';
b = b(:)';
if nargin < 2
       error('Not enough input for GFMUL')
elseif nargin < 3
    p = 2;
end
[n_p, m_p] = size(p);
if n_p <= 1
    %case of polynomial calculation
    if ~isempty([find(a~=floor(a)), find(a<0), find(a>=p)])
         error('The polynomial coeficients must be in GF(P)')
    elseif ~isempty([find(b~=floor(b)), find(b<0), find(b>=p)])
     error('The polynomial coeficients must be in GF(P)')
    end;

    %variable assignment
    len_a = length(a);
    len_b = length(b);
    if (len_a == len_b) | (len_b == 1)
        c = rem(a .* b, p);
    elseif len_a == 1
        c = rem(b .* a, p);
    elseif len_a > len_b
        c = a;
        c(1:len_b) = rem(a(1:len_b) .* b, p);
    else
        c = b;
        c(1:len_a) = rem(b(1:len_a) .* a, p);
    end;
else
    prim = max(max(p)) + 1;
    indx = find(a < 0);
    a(indx) = indx - Inf;
    indx = find(b < 0);
    b(indx) = indx - Inf;
    len_a = length(a);
    len_b = length(b);
    if (len_a == len_b) | (len_a == 1) | (len_b == 1)
        sum_ab = rem(a + b, n_p - 1);
        cal_len = max(len_a, len_b);
    else
        cal_len = min(len_a, len_b);
        sum_ab = rem(a(1 : cal_len) + b(1 : cal_len), n_p - 1);
    end;
    indx = [find(sum_ab < 0) find(isnan(sum_ab))];
    
    sum_ab(indx) = - ones(1, length(indx));
    tmp = p(sum_ab + 2, :);
    for i = 1: cal_len
        indx = find(sum(rem(p + prim - ones(n_p, 1) * tmp(i,:), prim)') == 0);
        if length(indx) == 1
            c(i) = indx - 2;
        else
            if isempty(indx)
                error('The list of Galois field is not a complete one');
            else
                error('The list of Galois field has repeat elements');
            end;
        end;
    end;
    indx = find(c < 0);
    if ~isempty(indx)
        c(indx) = indx - Inf;
    end;
end;
c = c(:);
%--end of GFMUL.M---
