/*
 * SRANDPOI  SIMULINK poisson distribution number generator.
 *
 *      Syntax:  [sys, x0] = SRANDPOI(t, x, u, flag, seed, lambda)
 *      This function is to generate poisson distribution number.
 *      When seed is a vector, the function generate a vecotr with
 *      the vector length same as the length of seed vector.
 *
 * Wes Wang  Feb. 16, 1995
 * Copyright (c) 1994-96 by The MathWorks, Inc.
 * All Rights Reserved
 * $Revision: 1.2 $  $Date: 1996/04/10 18:23:39 $
 */

#define S_FUNCTION_NAME srandpoi

#include <math.h>

#ifdef MATLAB_MEX_FILE
#include <stdio.h>    /* needed for declaration of sprintf */
#include "mex.h"      /* needed for declaration of mexErrMsgTxt */
#endif

/*
 * need to include simstruc.h for the definition of the SimStruct and
 * its associated macro definitions.
 */

#include "simstruc.h"

/*
 * Defines for easy access of the input parameters
 */

#define NUM_ARGS       2
#define INISEED        ssGetArg(S,0)
#define LAMBDA         ssGetArg(S,1)

#define LOW_SEED   1          /* minimum seed value */
#define HIGH_SEED  2147483646 /* maximum seed value */
#define START_SEED 1144108930 /* default seed value */

/*
 * mdlInitializeSizes - called to initialize the sizes array stored in
 *                      the SimStruct.  The sizes array defines the
 *                      characteristics (number of inputs, outputs,
 *                      states, etc.) of the S-Function.
 */

static double urand(seed)
	unsigned int *seed;
{
	unsigned int hi, lo;
#if (UINT_MAX == 0xffffffff)
	int test;
#else
	long test;
#endif

#define IA 16807	/* magic multiplier = 7^5 */
#define IM 2147483647	/* modulus = 2^31 - 1 */
#define IQ 127773	/*IM div IA */
#define IR 2836	/* IM modulo IA */
#define S  4.656612875245797e-10	/* reciprocal of 2^31-1 */

	hi = *seed / IQ;
	lo = *seed % IQ;
	test = IA * lo - IR * hi;

	*seed = ((test < 0) ? (unsigned int)(test + IM) : (unsigned int)test);

	return ((double) (*seed * S));

#undef IA
#undef IM
#undef IQ
#undef IR
#undef S
}

static void mdlInitializeSizes(S)
  SimStruct *S;
{
    /*
     * Set-up size information.
     */ 
    
    if (ssGetNumArgs(S) == NUM_ARGS) {
	int numOutput, iniSeed;

	numOutput = mxGetN(INISEED) * mxGetM(INISEED);
	if (numOutput < 1) {
#ifdef MATLAB_MEX_FILE
	    char err_msg[256];
	    sprintf(err_msg, "Input variable is empty.");
	    mexErrMsgTxt(err_msg);
#endif    
        }

	if (((mxGetN(LAMBDA) * mxGetM(LAMBDA)) != 1) && (mxGetN(LAMBDA) * mxGetM(LAMBDA)) != numOutput) {
#ifdef MATLAB_MEX_FILE
	    char err_msg[256];
	    sprintf(err_msg, "Dimension for LAMBDA must match the dimension for seeds.");
	    mexErrMsgTxt(err_msg);
#endif    
	}

    ssSetNumContStates(    S, 0);
	ssSetNumDiscStates(    S, 0);
	ssSetNumInputs(        S, 0);
	ssSetNumOutputs(       S, numOutput);
	ssSetDirectFeedThrough(S, 0);
	ssSetNumInputArgs(     S, NUM_ARGS);
	ssSetNumSampleTimes(   S, 1);
	ssSetNumRWork(         S, numOutput);
	ssSetNumIWork(         S, numOutput);
	ssSetNumPWork(         S, 0);
    } else {
#ifdef MATLAB_MEX_FILE
	char err_msg[256];
	sprintf(err_msg, "Wrong number of input arguments passed to S-function MEX-file.\n"
		"%d input arguments were passed in when expecting %d input arguments.\n", ssGetNumArgs(S) + 4, NUM_ARGS + 4);
	mexErrMsgTxt(err_msg);
#endif
    }
}

/*
 * mdlInitializeSampleTimes - initializes the array of sample times stored in
 *                            the SimStruct associated with this S-Function.
 */

static void mdlInitializeSampleTimes(S)
  SimStruct *S;
{
    /*
     * Note, blocks that are continuous in nature should have a single
     * sample time of 0.0.
     */

    ssSetSampleTimeEvent(S, 0, 0.0);
    ssSetOffsetTimeEvent(S, 0, 0.0);
}

/*
 * mdlInitializeConditions - initializes the states for the S-Function
 */

static void mdlInitializeConditions(x0, S)
  double *x0;
  SimStruct *S;
{
    int     numOutput, i, mFlag, while_flag;
    double  p, lam;
    double  *RWork         = ssGetRWork(S);       /* Real Work Vector */
    int     *IWork         = ssGetIWork(S);       /* Integer Work Vector */
    double  *Pseed         = mxGetPr(INISEED);
    double  *Lambda        = mxGetPr(LAMBDA);

    numOutput = mxGetN(INISEED) * mxGetM(INISEED);

    if (mxGetN(LAMBDA) * mxGetM(LAMBDA) <= 1) {
      mFlag = 1;
    } else {
      mFlag = 0;
    }

    for (i = 0; i < numOutput; i++) {
      IWork[i] = RWork[i];
      if ((IWork[i] < LOW_SEED) || (IWork[i] > HIGH_SEED))
	  IWork[i] = START_SEED + i;
      RWork[i] = 0;
      if (mFlag)
	lam = Lambda[0];
      else
	lam = Lambda[i];
      p = 0;
      while_flag = 1;
      while (while_flag) {
	p = p - log(urand((unsigned int *) &IWork[i]));
	if (p > lam)
	  while_flag = 0;
	else
	  RWork[i] += 1;
      }
    }
}

/*
 * mdlOutputs - computes the outputs of the S-Function
 */

static void mdlOutputs(y, x, u, S, tid)
  double *y, *x, *u;
  SimStruct *S; 
  int tid;
{
    int     numOutput, i, mFlag, while_flag;
    double  *RWork         = ssGetRWork(S);       /* Real Work Vector */
    int     *IWork         = ssGetIWork(S);       /* Integer Work Vector */
    double  *Lambda        = mxGetPr(LAMBDA);
    double  p, lam;

    numOutput = mxGetN(INISEED) * mxGetM(INISEED);

    if (mxGetN(LAMBDA) * mxGetM(LAMBDA) <= 1) {
      mFlag = 1;
    } else {
      mFlag = 0;
    }

    for (i = 0; i < numOutput; i++) {
      y[i] = RWork[i];
      RWork[i] = 0;
      if (mFlag)
	lam = Lambda[0];
      else
	lam = Lambda[i];
      p = 0;
      while_flag = 1;
      while (while_flag) {
	p = p - log(urand((unsigned int *) &IWork[i]));
	if (p > lam)
	  while_flag = 0;
	else
	  RWork[i] += 1;
      }
    }
}

/*
 * mdlUpdate - computes the discrete states of the S-Function
 */

static void mdlUpdate(x, u, S, tid)
  double *x, *u;
  SimStruct *S;
  int tid;
{
}

/*
 * mdlDerivatives - computes the derivatives of the S-Function
 */

static void mdlDerivatives(dx, x, u, S, tid)
  double *dx, *x, *u;
  SimStruct *S;
  int tid;
{
}

/*
 * mdlTerminate - called at termination of model execution.
 */

static void mdlTerminate(S)
  SimStruct *S;
{
}

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-File interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
