function a = appcoef(c,l,arg3,arg4,arg5)
%APPCOEF Extract 1-D approximation coefficients.
%       APPCOEF computes the approximation coefficients of a
%       one-dimensional signal.
%
%       A = APPCOEF(C,L,'wname',N) computes the approximation
%       coefficients at level N using the wavelet decomposition
%       structure [C,L] (see WAVEDEC).
%       'wname' is a string containing the wavelet name.
%       Level N must be an integer such that 0 <= N <= length(L)-2. 
%
%       A = APPCOEF(C,L,'wname') extracts the approximation
%       coefficients at the last level length(L)-2.
%
%       Instead of giving the wavelet name, you can give the filters.
%       For A = APPCOEF(C,L,Lo_R,Hi_R) or
%           A = APPCOEF(C,L,Lo_R,Hi_R,N),
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter.
%       
%       See also DETCOEF, WAVEDEC.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:05:26 $

%       Uses ERRARGN, ERRARGT, DETCOEF, IDWT, WFILTERS.

% Check arguments.
if errargn('appcoef',nargin,[3:5],nargout,[0:1]), error('*'), end
rmax = length(l);
nmax = rmax-2;
if isstr(arg3)
        [LoF_R,HiF_R] = wfilters(arg3,'r');
        if nargin == 4, n = arg4; else, n = nmax; end
else
        LoF_R = arg3;   HiF_R = arg4;
        if nargin == 5, n = arg5; else, n = nmax; end
end
if (n < 0) | (n > nmax) | (n ~= fix(n))
        errargt('appcoef','invalid level value','msg'); error('*');
end

% Initialization.
a = c(1:l(1));

% Iterated reconstruction.
imax = rmax+1;
for p = nmax:-1:n+1
        d = detcoef(c,l,p);                    % extract detail
        a = idwt(a,d,LoF_R,HiF_R,l(imax-p));
end
