function a = appcoef2(c,s,arg3,arg4,arg5)
%APPCOEF2 Extract 2-D approximation coefficients.
%       APPCOEF2 computes the approximation coefficients of a
%       two-dimensional signal.
%
%       A = APPCOEF2(C,S,'wname',N) computes the approximation
%       coefficients at level N using the wavelet decomposition
%       structure [C,S] (see WAVEDEC2). 
%       'wname' is a string containing the wavelet name.
%       Level N must be an integer such that 0 <= N <= size(S,1)-2. 
%
%       A = APPCOEF2(C,S,'wname') extracts the approximation
%       coefficients at the last level size(S,1)-2.
%
%       Instead of giving the wavelet name, you can give the filters.
%       For A = APPCOEF2(C,S,Lo_R,Hi_R) or 
%           A = APPCOEF2(C,S,Lo_R,Hi_R,N),
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter.
%       
%       See also DETCOEF2, WAVEDEC2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:05:31 $

%       Uses ERRARGN, ERRARGT, DETCOEF2, IDWT2, WFILTERS.

% Check arguments.
if errargn('appcoef2',nargin,[3:5],nargout,[0:1]), error('*'), end
rmax = size(s,1);
nmax = rmax-2;
if isstr(arg3)
        [LoF_R,HiF_R] = wfilters(arg3,'r');
        if nargin==4, n = arg4; else, n = nmax; end
else
        LoF_R = arg3;       HiF_R = arg4;
        if nargin==5, n = arg5; else, n = nmax; end
end

if (n<0) | (n>nmax) | (n~=fix(n))
        errargt('appcoef2','invalid level value','msg'); error('*');
end

nl = s(1,1);
nc = s(1,2);
a  = reshape(c(1:nl*nc),nl,nc);

rm = rmax+1;
for p=nmax:-1:n+1
        h = detcoef2('h',c,s,p);
        v = detcoef2('v',c,s,p);
        d = detcoef2('d',c,s,p);
        a = idwt2(a,h,v,d,LoF_R,HiF_R,s(rm-p,:));
end
