function [tree,data,ento,n2m] = bestlevt(tree,data)
%BESTLEVT Best level tree (wavelet packet).
%       BESTLEVT computes the optimal complete sub-tree of an
%       initial tree with respect to an entropy type criterion.
%       The resulting complete tree may be of smaller depth
%       than the initial one.
%
%       [T,D] = BESTLEVT(T,D) computes the modified tree
%       structure T and data structure D, corresponding to
%       the best level tree decomposition.
%
%       [T,D,E] = BESTLEVT(T,D) returns best tree T,
%       data structure D and in addition, the best entropy value E.
%
%       See also BESTTREE, MAKETREE, WENTROPY, WPDEC, WPDEC2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:05:36 $

%       Uses ERRARGN, DEPO2IND, IND2DEPO, WCOMMON, WDATAMGR, 
%            WPJOIN, WTREEMGR.

% Check arguments.
if errargn('bestlevt',nargin,[2],nargout,[2:4]), error('*'); end

order        = wtreemgr('order',tree);
an           = wtreemgr('allnodes',tree);
tn           = wtreemgr('tnodes',tree);

[dtn,ptn]    = ind2depo(order,tn);
[dan,pan]    = ind2depo(order,an);
dmin         = min(dtn);
K            = find(dan==dmin);
an_min       = depo2ind(order,[dan(K) pan(K)]);
[ind_an,nul] = wcommon(an_min,tn);

n2m          = [an_min(find(1-ind_an))];
od           = order^dmin;
nodes        = depo2ind(order,[dmin*ones(od,1) [0:od-1]']);
ento         = sum(wdatamgr('read_ent',data,nodes));
dini         = dmin;
for d=dini-1:-1:0
       od    = order^d;
       nodes = depo2ind(order,[d*ones(od,1) [0:od-1]']);
       ent   = sum(wdatamgr('read_ent',data,nodes));
       if ent<=ento
              dmin   = d;
              ento   = ent;
              n2m    = nodes;
       end;
end

for n = n2m(:)'
       [tree,data] = wpjoin(tree,data,n);
end
