function coefs = cwt(signal,scales,wname,flag);
%CWT Continuous wavelet coefficients 1-D.
%       COEFS = CWT(S,SCALES,'wname') computes the continuous
%       wavelet coefficients of the vector S at real, positive
%       SCALES, using wavelet whose name is 'wname'.
%
%       COEFS = CWT(S,SCALES,'wname','plot') computes,
%       and in addition plots, the continuous wavelet
%       transform coefficients.
%
%       For each given scale a within the vector SCALES, the  
%       wavelet coefficients C(a,b) are computed for b = 1 to
%       ls = length(S) and are stored in COEFS(i,:)
%       if a = SCALES(i). 
%       Output argument COEFS is a la-by-ls matrix, where la 
%       is the length of SCALES.
%
%       Examples of valid uses are:
%       c = CWT(s,1:32,'meyr')
%       c = CWT(s,[64 32 16:-2:2],'morl')
%       c = CWT(s,[3 18 12.9 7 1.5],'db2')
%
%       See also WAVEDEC, WAVEFUN, WAVEINFO, WCODEMAT.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:06:36 $

%       Uses ERRARGN, ERRARGT, INTWAVE, WCODEMAT, WKEEP.

% Check arguments.
if errargn('cwt',nargin,[3:4],nargout,[0 1]), error('*'), end

err     = 0;
if      isempty(scales)
        err = 1;
elseif  min(size(scales))>1
        err = 1;
elseif  min(scales)<eps
        err = 1;
end
if err
        errargt('cwt','Invalid Value for Scales !','msg');
        error('*')
end

signal                = signal(:)';
[psi_integ,psi_xval]  = intwave(wname);
psi_xval              = psi_xval-min(psi_xval);
dxval                 = psi_xval(2);
xmax                  = psi_xval(length(psi_xval));
len                   = length(signal);
coefs                 = zeros(length(scales),len);
ind                   = 1;
for a = scales
        j = [1+floor([0:a*xmax]/(a*dxval))];
        if length(j)==1 , j = [1 1]; end
        f            = fliplr(psi_integ(j));
        coefs(ind,:) = -sqrt(a)*wkeep(diff(conv(signal,f)),len);
        ind          = ind+1;
end
if nargin==4
        len   = length(scales);
        nb    = min(5,len);
        level = '';
        for k=1:nb , level = [level ' '  num2str(scales(k))]; end
        if nb<len , level = [level ' ...']; end
        NBC   = 128;
        img   = image(flipud(wcodemat(coefs,NBC)));
        len   = length(scales);
        nb    = ceil(len/20);
        tics  = len:-nb:1;
        tmp  = scales(1:nb:nb*length(tics));
        labs = num2str(tmp(1));
        for k = 2:length(tmp)
             labs = str2mat(labs,num2str(tmp(k)));
        end
        set(get(img,'parent'),              ...
                'YTicklabelMode','manual',  ...
                'YTick',tics,               ...
                'YTickLabels',labs,         ...
                'Box','On'                  ...
                );
        title(['Absolute Values of Ca,b Coefficients for a = ' level]);
        xlabel('time (or space) b');
        ylabel('scales a');
        colormap(cool(NBC));
end

