function [a,d] = dwt(x,arg2,arg3)
%DWT Single-level discrete 1-D wavelet transform.
%       DWT performs a single-level 1-D wavelet decomposition
%       with respect to either a particular wavelet ('wname',
%       see WFILTERS) or particular wavelet filters
%       (Lo_D and Hi_D) you specify.
%
%       [CA,CD] = DWT(X,'wname') computes the approximation
%       coefficients vector CA and detail coefficients vector CD,
%       obtained by wavelet decomposition of the vector X.
%       'wname' is a string containing the wavelet name.
%
%       [CA,CD] = DWT(X,Lo_D,Hi_D) computes the wavelet decomposition
%       as above given these filters as input:
%       Lo_D is the decomposition low-pass filter and
%       Hi_D is the decomposition high-pass filter.
%       Lo_D and Hi_D must be the same length.
%
%       If lx = length(X) and lf is the length of filters then
%       length(CA) = length(CD) = FLOOR((lx+lf-1)/2).
%
%       For the different signal extension modes, see DWTMODE. 
%
%       See also DWTMODE, DWTPER, IDWT, WAVEDEC, WAVEINFO.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:10:01 $

%       Uses DYADDOWN, WKEEP, WFILTERS.

global DWT_Ext_Mode

% Check arguments.
if errargn('dwt',nargin,[2:3],nargout,[0:2]), error('*'), end
if nargin == 2
        [LoF_D,HiF_D] = wfilters(arg2,'d');
else
        LoF_D = arg2;   HiF_D = arg3;
end

if      isempty(DWT_Ext_Mode) | DWT_Ext_Mode=='zpd' % zpd is the default
        % Decomposition.
        a = dyaddown(conv(x,LoF_D));
        d = dyaddown(conv(x,HiF_D));

elseif  DWT_Ext_Mode=='sym'

        % Symmetrization.
        lx = length(x);
        lf = length(LoF_D);
        if size(x,1)==1
                x = [x(lf:-1:1) x x(lx:-1:lx-lf+1)];
        else
                x = [x(lf:-1:1); x ; x(lx:-1:lx-lf+1)];
        end

        % Decomposition.
        la = floor((lx+lf-1)/2);
        a  = wkeep(dyaddown(conv(x,LoF_D)),la);
        d  = wkeep(dyaddown(conv(x,HiF_D)),la);

elseif  DWT_Ext_Mode=='spd'

        % Smooth padding.
        lx = length(x);
        lf = length(LoF_D);
        if size(x,1)==1
                x = [zeros(1,lf) x zeros(1,lf)];
        else
                x = [zeros(lf,1); x ; zeros(lf,1)];
        end
        for k=lf:-1:1
                x(k) = 2*x(k+1)-x(k+2);
        end
        for k=lf+lx+1:lx+2*lf
                x(k) = 2*x(k-1)-x(k-2);
        end

        % Decomposition.
        la = floor((lx+lf-1)/2);
        a  = wkeep(dyaddown(conv(x,LoF_D)),la);
        d  = wkeep(dyaddown(conv(x,HiF_D)),la);

else
        errargt('dwt','Invalid Extension Mode for DWT!','msg');
        error('*')

end
