function dwtmode(in1,in2)
%DWTMODE Discrete wavelet transform extension mode.
%       DWTMODE sets the signal or image extension mode for
%       discrete wavelet and wavelet packets transforms.
%       The extension modes represent different ways of handling
%       the problem of border distortion in the analysis.
%
%       DWTMODE or DWTMODE('status') displays the current mode.
%       DWTMODE('zpd') sets the DWT mode to zero padding
%              (default mode).
%       DWTMODE('sym') sets the DWT mode to symmetrization
%              (boundary value replication).
%       DWTMODE('spd') sets the DWT mode to smooth padding
%              (first derivative interpolation at the edges).
%
%       If DWTMODE is called with two input arguments, the second
%       one is dummy and no text (status or warning) is displayed
%       in the matlab command window.
%
%       This function updates a global variable allowing
%       three ways of signal extension. 
%       Only dwt and dwt2 use the global variable.
%
%       You should change the extension mode only using
%       DWTMODE; avoide changing the global variable directly.
%
%       See also DWT, DWT2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:10:11 $

%       Uses ERRARGN, ERRARGT.

global DWT_Ext_Mode

% Check arguments.
if errargn('dwtmode',nargin,[0 1 2],nargout,[0]), error('*'), end
if nargin==0
        in1 = 'status';
end
in1 = lower(in1);
if strcmp(in1,'status')
        if ~isempty(DWT_Ext_Mode)
                in1 = DWT_Ext_Mode;
        else
                in1 = 'zpd';
                DWT_Ext_Mode = in1;
        end
else
        if     strcmp(in1,'zpd')
        elseif strcmp(in1,'sym')
        elseif strcmp(in1,'spd')
        else
                errargt('dwtmode','Unknown Extension Mode','msg');
                error('*');
        end
        if      isempty(DWT_Ext_Mode)
                DWT_Ext_Mode ='zpd';
        elseif  any(DWT_Ext_Mode~=in1)
                DWT_Ext_Mode = in1;
                if nargin<2
                   disp(' ');
                   disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
                   disp('!  WARNING: Change DWT Extension Mode  !');
                   disp('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!');
                end
        end
end

if      strcmp(in1,'zpd')
        msg = 'DWT Extension Mode: Zero Padding';
elseif  strcmp(in1,'sym')
        msg = 'DWT Extension Mode: Symmetrization';
elseif  strcmp(in1,'spd')
        msg = 'DWT Extension Mode: Smooth Padding';
end

l = length(msg)+8;
c = '*';
s = c(ones(1,l));
if nargin<2
   disp(' ');
   disp(s);
   disp([c c '  ' msg '  ' c c])
   disp(s);
   disp(' ');
end

