function y = dyaddown(x,arg2,arg3)
%DYADDOWN Dyadic downsampling.
%       Y = DYADDOWN(X,EVENODD), where X is a vector, returns
%       a version of X that has been downsampled by 2.
%       Whether Y contains the even- or odd-indexed samples
%       of X depends on the value of positive integer EVENODD:
%       If EVENODD is even, then Y(k) = X(2k).
%       If EVENODD is odd,  then Y(k) = X(2k+1).
%
%       Y = DYADDOWN(X) is equivalent to Y = DYADDOWN(X,0)
%
%       Y = DYADDOWN(X,EVENODD,'type') or 
%       Y = DYADDOWN(X,'type',EVENODD) where X is a matrix,
%       return a version of X obtained by suppressing columns
%       (or rows or both) if 'type' = 'c' (or 'r' or 'm'
%       respectively), according to the parameter EVENODD which
%       is as above.
%
%       Y = DYADDOWN(X) is equivalent to
%           Y = DYADDOWN(X,0,'c').
%       Y = DYADDOWN(X,'type') is equivalent to
%           Y = DYADDOWN(X,0,'type').
%       Y = DYADDOWN(X,EVENODD) is equivalent to
%           Y = DYADDOWN(X,EVENODD,'c').
%
%                      |1 2 3 4|                       |2 4|
%       Examples : X = |2 4 6 8|  ,  DYADDOWN(X,'c') = |4 8|
%                      |3 6 9 0|                       |6 0|
%
%                           |1 2 3 4|                        |1 3|
%       DYADDOWN(X,'r',1) = |3 6 9 0|  , DYADDOWN(X,'m',1) = |3 9|
%
%       See also DYADUP.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:10:27 $

% Check arguments.
if errargn('dyaddown',nargin,[1:3],nargout,[0:1]), error('*'), end

[r,c] = size(x);
if min(r,c)<=1
        if     nargin==1, p = 0;
        elseif nargin==2, p = arg2;
        else   errargt('dyaddown','too many arguments','msg'); error('*');
        end
        if rem(p,2), y = x(1:2:length(x)); else, y = x(2:2:length(x)); end
else
        if     nargin==1, o = 'c'; p = 0;
        elseif nargin==2
                if isstr(arg2), p = 0; o = lower(arg2(1));
                else, p = arg2; o = 'c';
                end
        else
                if isstr(arg2), p = arg3; o = lower(arg2(1));
                else, p = arg2; o = lower(arg3(1));
                end
        end
        if     o=='c'
                if rem(p,2), y = x(:,1:2:c);
                else, y = x(:,2:2:c); 
                end
        elseif o=='r'
                if rem(p,2), y = x(1:2:r,:);
                else, y = x(2:2:r,:); 
                end
        elseif o=='m'
                if rem(p,2), y = x(1:2:r,1:2:c);
                else, y = x(2:2:r,2:2:c); 
                end
        else
                errargt('dyaddown','invalid argument value','msg'); error('*');
        end
end
