function y = dyadup(x,arg2,arg3)
%DYADUP Dyadic upsampling.
%       DYADUP implements a simple zero-padding scheme very
%       useful in the wavelet reconstruction algorithm.
%
%       Y = DYADUP(X,EVENODD), where X is a vector, returns
%       an extended copy of vector X obtained by inserting zeros.
%       Wether the zeros are inserted as even- or odd-indexed
%       elements of Y depends on the value of psitive integer
%       EVENODD:
%       If EVENODD is even, then Y(2k+1) = X(k), Y(2k) = 0.
%       If EVENODD is odd,  then Y(2k+1) = 0   , Y(2k) = X(k).
%
%       Y = DYADUP(X) is equivalent to Y = DYADUP(X,1)
%
%       Y = DYADUP(X,EVENODD,'type') or 
%       Y = DYADUP(X,'type',EVENODD) where X is a matrix,
%       return extended copies of X obtained by inserting columns 
%       of zeros (or rows or both) if 'type' = 'c' (or 'r' or 'm'
%       respectively), according to the parameter EVENODD which
%       is as above.
%
%       Y = DYADUP(X) is equivalent to
%           Y = DYADUP(X,1,'c')
%       Y = DYADUP(X,'type')  is equivalent to
%           Y = DYADUP(X,1,'type')
%       Y = DYADUP(X,EVENODD) is equivalent to
%           Y = DYADUP(X,EVENODD,'c') 
%
%                      |1 2|                     |0 1 0 2 0|
%       Examples : X = |3 4|  ,  DYADUP(X,'c') = |0 3 0 4 0|
%
%                         |1 2|                      |1 0 2|
%       DYADUP(X,'r',0) = |0 0|  , DYADUP(X,'m',0) = |0 0 0|
%                         |3 4|                      |3 0 4|
%
%       See also DYADDOWN.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:10:32 $

% Check arguments.
if errargn('dyadup',nargin,[1:3],nargout,[0:1]), error('*'), end

[r,c] = size(x);
mi    = min(r,c);
dim   = 1;
if mi>1
        dim = 2;
else
        if     nargin==2
               if isstr(arg2) , dim =2; end
        elseif nargin==3
               if isstr(arg2) | isstr(arg3) , dim = 2; end
        end
end
if dim==1
        if     nargin==1, p = 1;
        elseif nargin==2, p = arg2;
        else   errargt('dyadup','too many arguments','msg'); error('*');
        end
        if rem(p,2)
                l = 2*length(x)+1; y  = zeros(1,l); y(2:2:l) = x;
                if r>1, y = y'; end
        else
                l = 2*length(x)-1; y  = zeros(1,l); y(1:2:l) = x;
                if r>1, y = y'; end
        end
else
        if     nargin==1, p = 1; o = 'c';
        elseif nargin==2
                if isstr(arg2), p = 1; o = lower(arg2(1));
                else p = arg2; o = 'c';
                end
        else
                if isstr(arg2), p = arg3; o = lower(arg2(1));
                else, p = arg2; o = lower(arg3(1));
                end
        end
        if     o=='c'
                if rem(p,2)
                        nc = 2*c+1; y  = zeros(r,nc); y(:,2:2:nc) = x;
                else
                        nc = 2*c-1; y  = zeros(r,nc); y(:,1:2:nc) = x;
                end

        elseif o=='r'
                if rem(p,2)
                        nr = 2*r+1; y  = zeros(nr,c); y(2:2:nr,:) = x;
                else
                        nr = 2*r-1; y  = zeros(nr,c); y(1:2:nr,:) = x;
                end
        elseif o=='m'
                if rem(p,2)
                        nr = 2*r+1; nc = 2*c+1;
                        y  = zeros(nr,nc); y(2:2:nr,2:2:nc) = x;
                else
                        nr = 2*r-1; nc = 2*c-1; 
                        y  = zeros(nr,nc); y(1:2:nr,1:2:nc) = x;
                end
        else
                errargt('dyadup','invalid argument value','msg'); error('*');
        end
end

