function x = idwt2(a,h,v,d,arg5,arg6,arg7)
%IDWT2  Single-level inverse discrete 2-D wavelet transform.
%       IDWT2 performs a single-level 2-D wavelet reconstruction
%       with respect to either a particular wavelet
%       ('wname', see WFILTERS) or particular wavelet
%       reconstruction filters you specify.
%
%       X = IDWT2(CA,CH,CV,CD,'wname') uses the wavelet 'wname'
%       to compute the single-level reconstructed approximation
%       coefficients matrix X based on approximation matrix CA
%       and (horizontal, vertical, and diagonal) details matrices
%       CH, CV and CD.
%
%       X = IDWT2(CA,CH,CV,CD,Lo_R,Hi_R) reconstructs as above,
%       using filters you specify:
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter.
%       Lo_R and Hi_R must be the same length.
%
%       If sa = size(CA) = size(CH) = size(CV) = size(CD) and
%       lf is the length of the filters, then
%       size(X) = 2*size(CA)-lf+2.
%
%       X = IDWT2(CA,CH,CV,CD,'wname',S) and
%       X = IDWT2(CA,CH,CV,CD,Lo_R,Hi_R,S) return the size-S
%       central portion of the result obtained using
%       IDWT2(CA,CH,CV,CD,'wname'). S must be less than
%       2*size(CA)-lf+2.
%
%       See also DWT2, IDWTPER2, UPWLEV2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:11:37 $

%       Uses ERRARGN, DYADUP, WFILTERS, WKEEP.

% Check arguments.
if errargn('idwt2',nargin,[5:7],nargout,[0 1]), error('*'), end
if isstr(arg5)
        [LoF_R,HiF_R] = wfilters(arg5,'r');             
        if nargin==6, s = arg6; else, s = 2*size(a)-length(LoF_R)+2; end 
else
        LoF_R = arg5;   HiF_R = arg6;   
        if nargin==7, s = arg7; else, s = 2*size(a)-length(LoF_R)+2; end
end

% Reconstruction.
t0 = conv2(dyadup(a,'r'),LoF_R')+conv2(dyadup(h,'r'),HiF_R');
clear a h
d0 = conv2(dyadup(v,'r'),LoF_R')+conv2(dyadup(d,'r'),HiF_R');
clear v d
x = conv2(dyadup(t0,'c'),LoF_R)+conv2(dyadup(d0,'c'),HiF_R);
x = wkeep(x,s);

