function x = idwtper2(a,h,v,d,arg5,arg6,arg7)
%IDWTPER2 Single-level inverse discrete 2-D wavelet transform (periodized).
%       X = IDWTPER2(CA,CH,CV,CD,'wname') returns the single-level
%       reconstructed approximation coefficients matrix X based on
%       approximation and details matrices CA, CH, CV and CD at a 
%       given level, using the periodized inverse wavelet transform.
%       'wname' is a string containing the wavelet name
%       (see WFILTERS).
%
%       Instead of giving the wavelet name, you can give the
%       filters.
%       For X = IDWTPER2(CA,CH,CV,CD,Lo_R,Hi_R), 
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter
%
%       If sa = size(CA) = size(CH) = size(CV) = size(CD) then
%       size(X) = 2*sa.
%
%       For X = IDWTPER2(CA,CH,CV,CD,'wname',S) or
%       X = IDWTPER2(CA,CH,CV,CD,Lo_R,Hi_R,S), S is the size of
%       the result.
%
%       See also DWTPER2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:11:48 $

%       Uses DYADUP, ERRARGN, WFILTERS, WKEEP.

% Check arguments.
if errargn('idwtper2',nargin,[5:7],nargout,[0 1]), error('*'), end
sa = size(a);
if isstr(arg5)
        [LoF_R,HiF_R] = wfilters(arg5,'r');             
        if nargin==6, sx = arg6; else, sx = 2*sa; end
else
        LoF_R = arg5;   HiF_R = arg6;   
        if nargin==7, sx = arg7; else, sx = 2*sa; end
end

% Reconstruction.
lf = length(LoF_R);
sp = ceil(sx/2);        rp = sp(1);        cp = sp(2);
sm = (floor((sx+lf-1)/2)-sp)/2;
ard = ceil(sm);         rrd = ard(1);      crd = ard(2);
alu = floor(sm);        rlu = alu(1);      clu = alu(2);

a = [a(:,cp-clu+1:cp) a a(:,1:crd)];
a = [a(rp-rlu+1:rp,:);a;a(1:rrd,:)];

h = [h(:,cp-clu+1:cp) h h(:,1:crd)];
h = [h(rp-rlu+1:rp,:);h;h(1:rrd,:)];

v = [v(:,cp-clu+1:cp) v v(:,1:crd)];
v = [v(rp-rlu+1:rp,:);v;v(1:rrd,:)];

d = [d(:,cp-clu+1:cp) d d(:,1:crd)];
d = [d(rp-rlu+1:rp,:);d;d(1:rrd,:)];

t0 = conv2(dyadup(a,'r'),LoF_R')+conv2(dyadup(h,'r'),HiF_R');
clear a h
d0 = conv2(dyadup(v,'r'),LoF_R')+conv2(dyadup(d,'r'),HiF_R');
clear v d
x = conv2(dyadup(t0,'c'),LoF_R)+conv2(dyadup(d0,'c'),HiF_R);
x = wkeep(x,sx);

