function [x,t] = instdfft(xhat,lowb,uppb);
%INSTDFFT Inverse non-standard 1-D fast Fourier transform.
%       [X,T] = INSTDFFT(XHAT,LOWB,UPPB) returns the inverse
%       non-standard FFT of XHAT, on a power of 2 regular
%       grid (non necessarily integers) on the interval
%       [LOWB,UPPB].
%       Output arguments are X the recovered signal computed
%       on the time interval T given by
%       T = LOWB + [0:n-1]*(UPPB-LOWB)/n, where n is the
%       length of XHAT. Outputs are vectors of length n.
%
%       The length of XHAT must be a power of two.
%
%       See also FFT, IFFT, NSTDFFT.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:12:10 $

%       Uses ERRARGN, ERRARGT.

% Check arguments.
if errargn('instdfft',nargin,[3],nargout,[0:2]), error('*'), end
n = length(xhat);
if errargt('instdfft',log(n)/log(2),'int'), error('*'), end
if errargt('instdfft',uppb-lowb,'re0'), error('*'), end

% Time grid resolution.
delta = (uppb-lowb)/n;

% Frequency grid.
omega = [-n:2:n-2]/(2*n*delta);

% Transform back xhat to standard fft form.
xhat =  fftshift(xhat.*exp(2*pi*i*omega*lowb)/delta);

% Compute standard ifft.
x = ifft(xhat);

% Remove small imaginary parts.
sim = find(imag(x) < sqrt(eps));
if ~isempty(sim), x(sim) = real(x(sim)); end

% Time grid.
t = lowb + [0:n-1]*delta;

