function [out1,out2,out3] = meyer(lowb,uppb,n,o);
%MEYER Meyer wavelet.
%       [PHI,PSI,T] = MEYER(LOWB,UPPB,N) returns Meyer
%       wavelet and scaling functions evaluated on a N
%       regular grid on the interval [LOWB,UPPB].
%       Output arguments are the scaling function PHI, the
%       wavelet function PSI computed on the grid T.
%       These functions have [-8 8] as effective support.
%
%       A fourth argument is allowed if only one function
%       is required:
%       [PHI,T] = MEYER(LOWB,UPPB,N,'phi')
%       [PSI,T] = MEYER(LOWB,UPPB,N,'psi')
%       when the fourth argument is used but not equal to
%       'phi' or 'psi', outputs are the same as in main option.
%
%       N must be a power of two.
%
%       See also MEYERAUX, WAVEFUN, WAVEINFO.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:13:03 $

%       Uses ERRARGN, ERRARGT, INSTDFFT, MEYERAUX.

% Check arguments.
if errargn('meyer',nargin,[3:4],nargout,[0:3]), error('*'), end
if errargt('meyer',log(n)/log(2),'int')
        error('** Invalid value for n **')
end
if errargt('meyer',uppb-lowb,'re0'), error('*'), end

% Set default values.
if nargin==3
        o='two';
elseif ~strcmp(o,'phi') & ~strcmp(o,'psi')
        o='two';
end

% Transform interval bounds to grid.
lint = (uppb-lowb)/2/pi; 
x    = [-n:2:n-2]/(2*lint);
xa   = abs(x);

% Scaling function phi.
if o == 'phi' | o == 'two'

        % Compute support of Fourier transform of phi.
        int1 = find((xa < 2*pi/3));
        int2 = find((xa >= 2*pi/3) & (xa < 4*pi/3));

        % Compute Fourier transform of phi.
        phihat = zeros(1,n);
        phihat(int1) = ones(size(int1));
        phihat(int2) = cos(pi/2*meyeraux(3/2/pi*xa(int2)-1));

        % Compute phi using non standard inverse FFT.
        [phi,t] = instdfft(phihat,lowb,uppb);
end

% Wavelet function psi.
if o == 'psi' | o == 'two'

        % Compute support of Fourier transform of psi.
        int1 = find((xa >= 2*pi/3) & (xa < 4*pi/3)); 
        int2 = find((xa >= 4*pi/3) & (xa < 8*pi/3));

        % Compute Fourier transform of psi.
        psihat=zeros(1,n);
        psihat(int1) = exp(i*x(int1)/2).*sin(pi/2*meyeraux(3/2/pi*xa(int1)-1));
        psihat(int2) = exp(i*x(int2)/2).*cos(pi/2*meyeraux(3/4/pi*xa(int2)-1));

        % Compute psi using non standard inverse FFT.
        [psi,t] = instdfft(psihat,lowb,uppb);
end

% Set output arguments.
if     o == 'psi', out1=psi; out2=t;
elseif o == 'phi', out1=phi; out2=t;
else,  out1=phi; out2=psi; out3=t;
end

