function y = upcoef(o,x,arg3,arg4,arg5,arg6)
%UPCOEF Direct reconstruction from 1-D wavelet coefficients.
%       Y = UPCOEF(O,X,'wname',N) computes the N steps 
%       reconstructed coefficients of vector X. 'wname' is a
%       string containing the wavelet name.
%       N must be a strictly positive integer.
%       If O = 'a', approximation coefficients are reconstructed.
%       If O = 'd', detail coefficients are reconstructed.
%
%       Y = UPCOEF(O,X,'wname',N,L) computes the N steps 
%       reconstructed coefficients of vector X and takes the 
%       length-L central portion of the result.
%
%       Instead of giving the wavelet name, you can give the
%       filters.
%       For Y = UPCOEF(O,X,Lo_R,Hi_R,N) or 
%           Y = UPCOEF(O,X,Lo_R,Hi_R,N,L),
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter.
%
%       Y = UPCOEF(O,X,'wname') is equivalent to 
%       Y = UPCOEF(O,X,'wname',1).
%
%       Y = UPCOEF(O,X,Lo_R,Hi_R) is equivalent to 
%       Y = UPCOEF(O,X,Lo_R,Hi_R,1).
%                                 
%       In the last two cases, Y is not truncated.
%
%       See also IDWT.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:17:40 $

%       Uses ERRARGN, ERRARGT, DYADUP, WFILTERS, WKEEP.

% Check arguments.
if errargn('upcoef',nargin,[3:6],nargout,[0:1]), error('*'), end

o = lower(o(1));
if isstr(arg3)
        [LoF_R,HiF_R] = wfilters(arg3,'r');
        if nargin < 4, n = 1; l = 0;
        else
                n = arg4;
                if nargin < 5, l = 0; else l = arg5; end         
        end
else
        LoF_R = arg3;   HiF_R = arg4;
        if nargin < 5, n = 1; 
        else
                n = arg5;
                if nargin < 6, l = 0; else l = arg6; end         
        end
end
if (n<1) | (n~=fix(n))
        errargt('upcoef','invalid argument value','msg'); error('*');
end
if (l<0) | (l~=fix(l))
        errargt('upcoef','invalid argument value','msg'); error('*');
end

if     o=='a'
        % Approximation reconstruction.
        F1 = LoF_R;     F2 = F1;
elseif o=='d'
        % Detail reconstruction.
        F1 = HiF_R;     F2 = LoF_R;
else
        errargt('upcoef','invalid argument value','msg'); error('*');
end
y = conv(dyadup(x),F1);
for k=2:n
        y = conv(dyadup(y),F2);
end
if l, y = wkeep(y,l); end
