function y = upcoef2(o,x,arg3,arg4,arg5,arg6)
%UPCOEF2 Direct reconstruction from 2-D wavelet coefficients.
%       Y = UPCOEF2(O,X,'wname',N,S) computes the N steps 
%       reconstructed coefficients of matrix X and takes the
%       size-S central portion of the result.
%       'wname' is a string containing the name of the wavelet.
%       If O = 'a', approximation coefficients are reconstructed,
%       otherwise if O = 'h' (or 'v' or 'd'), horizontal
%       (vertical or diagonal respectively),
%       detail coefficients are reconstructed.
%       N must be a strictly positive integer.
%
%       Instead of giving the wavelet name, you can give the
%       filters.
%       For Y = UPCOEF2(O,X,Lo_R,Hi_R,N,S) 
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter.
%
%       Y = UPCOEF2(O,X,'wname',N) or Y = UPCOEF2(O,X,Lo_R,Hi_R,N),
%       return the computed result without any truncature. 
% 
%       Y = UPCOEF2(O,X,'wname') is equivalent to
%       Y = UPCOEF2(O,X,'wname',1).
%
%       Y = UPCOEF2(O,X,Lo_R,Hi_R) is equivalent to
%       Y = UPCOEF2(O,X,Lo_R,Hi_R,1).
%
%       See also IDWT2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:17:46 $

%       Uses ERRARGN, ERRARGT, DYADUP, WFILTERS, WKEEP.

% Check arguments.
if errargn('upcoef2',nargin,[3:6],nargout,[0 1]), error('*'), end

o = lower(o(1));
if isstr(arg3)
        [LoF_R,HiF_R] = wfilters(arg3,'r');
        if nargin < 4 
                n = 1; 
                s = [0 0];
        else
                n = arg4;
                if nargin < 5, s = [0 0]; else s = arg5; end     
        end
else
        LoF_R = arg3;   HiF_R = arg4;
        if nargin < 5 
                n = 1; 
                s = [0 0];
        else
                n = arg5;
                if nargin < 6, s = [0 0]; else s = arg6; end     
        end
end
if (n<0) | (n~=fix(n))
        errargt('upcoef2','invalid argument value','msg'); error('*');
end
if ~isempty(find(s<0)) | ~isempty(find(s~=fix(s)))
        errargt('upcoef2','invalid argument value','msg'); error('*');
end

if     strcmp(o,'a'),   f1 = LoF_R;     f2 = LoF_R;
elseif strcmp(o,'h'),   f1 = HiF_R;     f2 = LoF_R;
elseif strcmp(o,'v'),   f1 = LoF_R;     f2 = HiF_R;
elseif strcmp(o,'d'),   f1 = HiF_R;     f2 = HiF_R;
else,  errargt('upcoef2','invalid argument value','msg'); error('*');
end

y = conv2(dyadup(x,'r'),f1');
y = conv2(dyadup(y,'c'),f2);

for p = 2:n
        y = conv2(dyadup(y,'r'),LoF_R');
        y = conv2(dyadup(y,'c'),LoF_R);
end
if prod(s), y = wkeep(y,s); end;
