function [out1,out2,out3,out4,out5] = wavefun(wname,in2,in3);
%WAVEFUN Wavelet and scaling functions.
%       WAVEFUN returns approximations of the wavelet function
%       'wname' and the associated scaling function, if it exists.
%       Positive integer ITER is the number of iterations.
%
%       For an orthogonal wavelet:
%       [PHI,PSI,XVAL] = WAVEFUN('wname',ITER)  
%       returns the scaling and wavelet functions on the 2^ITER
%       points grid XVAL. 
%
%       For a biorthogonal wavelet:
%       [PHI1,PSI1,PHI2,PSI2,XVAL] = WAVEFUN('wname',ITER)
%       returns the scaling and wavelet functions both
%       for decomposition (PHI1, PSI1) and for
%       reconstruction (PHI2, PSI2). 
%
%       For a Meyer wavelet:
%       [PHI,PSI,XVAL] = WAVEFUN('wname',ITER)  
%
%       For a Morlet or Mexican Hat wavelet:
%       [PSI,XVAL] = WAVEFUN('wname',ITER) 
%
%       WAVEFUN('wname',A,B), where A and B are positive integers,
%       is equivalent to WAVEFUN('wname',max(A,B)), and plots are
%       produced.
%       When A is equal to the special value 0,
%       WAVEFUN('wname',0) is equivalent to WAVEFUN('wname',8,IN3).
%       WAVEFUN('wname')   is equivalent to WAVEFUN('wname',8).
%      
%       See also INTWAVE, WAVEINFO, WFILTERS.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:19:11 $

%       Uses DEBLANKL, MEXIHAT, MEYER, MORLET, UPCOEF, WFILTERS. 

% Check arguments.
if errargn('wavefun',nargin,[1:3],nargout,[0:5]), error('*'); end

wname = deblankl(wname);
debut = wname(1:2);
[wtype,fname] = wavemngr('info',wname);

if      nargin == 1,        iter = 8;        pflag = 0;
elseif  nargin == 2
        if in2 == 0,        iter = 8;        pflag = 1;
        else,               iter = in2;      pflag = 0;
        end
else,   iter = max(in2,in3); pflag = 1;
end

coef = (sqrt(2)^iter);
pas  = 1/(2^iter);

if      wtype==1
        draw_val = 1;
        [LoF_R,HiF_R] = wfilters(wname,'r');
        long  = length(LoF_R);
        phi   = coef*upcoef('a',1,LoF_R,HiF_R,iter);
        psi   = coef*upcoef('d',1,LoF_R,HiF_R,iter);

        lplus = long-2;
        nb = 1; for kk = 1:iter, nb = 2*nb+lplus; end

        phi = [0 wkeep(phi,nb) 0];
        psi = [0 wkeep(psi,nb) 0];

        % sign depends on wavelet 
        if strcmp(debut,'co') | strcmp(debut,'sy')  % coiflet
                psi = -psi ;                        % or symlet
        end
        out1 = phi;     out2 = psi;
        out3 = linspace(0,(nb+1)*pas,nb+2);

elseif  wtype==2
        draw_val = 2;
        [LoF_D1,HiF_D2,LoF_R2,HiF_R1,...
                LoF_D2,HiF_D1,LoF_R1,HiF_R2] = wfilters(wname);

        mul = 1;
        if debut=='bi'
                Nr = str2num(wname(5));
                if rem(Nr,4)~=1 , mul = -1; end
        end

        long   = length(LoF_D1);
        phi1   = coef*upcoef('a',1,LoF_R1,HiF_R1,iter);
        psi1   = mul*coef*upcoef('d',1,LoF_R1,HiF_R2,iter);

        lplus  = long-2;
        nb = 1; for kk = 1:iter, nb = 2*nb+lplus; end
        phi1   = [0 wkeep(phi1,nb) 0];
        psi1   = [0 wkeep(psi1,nb) 0];

        long   = length(LoF_R2);
        HiF_R2 = fliplr(HiF_D2);

        phi2   = coef*upcoef('a',1,LoF_R2,HiF_R2,iter);
        psi2   = mul*coef*upcoef('d',1,LoF_R2,HiF_R1,iter);

        lplus  = long-2;
        nb = 1;  for kk = 1:iter, nb = 2*nb+lplus; end
        phi2   = [0 wkeep(phi2,nb) 0];
        psi2   = [0 wkeep(psi2,nb) 0];
        out1   = phi1;    out2 = psi1;
        out3   = phi2;    out4 = psi2;
        out5   = linspace(0,(nb+1)*pas,nb+2);

elseif  wtype==3
        np = 2^iter;
        draw_val = 3;
        [lb,ub] = wavemngr('user',wname);
        [out1,out2,out3] = feval(fname,lb,ub,np);

elseif  wtype==4
        np = 2^iter;
        draw_val = 4;
        [lb,ub] = wavemngr('user',wname);
        [out1,out2] = feval(fname,lb,ub,np);
end

if pflag        % plots required.
        if      draw_val==1
                nb   = length(phi); xmax = nb*pas; xmin = 0;
                xval = linspace(xmin,xmax,nb);
                subplot(121);plot(xval,phi,'r');grid;
                title([wname ' : phi']);
                subplot(122);plot(xval,psi,'g');grid;
                title([wname ' : psi']); 

        elseif  draw_val==2
                nb   = length(phi1); xmax = nb*pas; xmin = 0;
                xval = linspace(xmin,xmax,nb);
                subplot(221);plot(xval,phi1,'r');grid;
                title([wname ' : phi dec.']);
                subplot(222);plot(xval,psi1,'g');grid;
                title([wname ' : psi dec.']);
                nb   = length(phi2); xmax = nb*pas; xmin = 0;
                xval = linspace(xmin,xmax,nb);
                subplot(223);plot(xval,phi2,'r');grid;
                title([wname ' : phi rec.']);
                subplot(224);plot(xval,psi2,'g');grid;
                title([wname ' : psi rec.']);

        elseif  draw_val==3
                fam = deblank(wavemngr('wfln',wname));
                subplot(121);plot(out3,out1,'r');grid;
                title([fam '  scaling function'])
                subplot(122);plot(out3,out2,'g');grid;
                title([fam '  wavelet function'])

        elseif  draw_val==4
                fam = deblank(wavemngr('wfln',wname));
                plot(out2,out1,'g');grid;
                title([fam '  wavelet function'])

        end
end
