function [thresh,rl2scr,n0scr,argmin,suggthr] = wcmpscr(c,l,in3)
%WCMPSCR 1D or 2D Wavelet Compression Scores.
%       [THR,RL2,NZ,IMIN,STHR] = WCMPSCR(C,L) returns for
%       the input wavelet decomposition structure [C,L],
%       compression scores for detail coefficients
%       thresholding and suggested threshold.
%       Outputs are :
%       THR the vector or ordered thresholds.
%       and vectors of scores induced by a THR(i)-thresholding :
%       RL2 vector of 2-norm recovery score in percent.
%       NZ  vector of relative number of zeros in percent.
%       IMIN is the index of THR for which the two scores are
%       approximately the same.
%       STHR is a suggested threshold.
%
%       When used with three arguments WCMPSCR(C,L,IN3) returns
%       the same outputs but for approximation and details
%       coefficients thresholding.
%
%       See also KEY2INFO, WPCMPSCR.
                    
%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:20:14 $

% Check arguments and set problem dimension.
if errargn('wcmpscr',nargin,[2:3],nargout,[0:5]), error('*'), end
dim = 1;        if min(size(l))~=1, dim = 2; end
keepapp = (nargin == 2);

% Set possible thresholds.
if keepapp      % only detail coeffs can be thresholded.
        if dim == 1
                app = c(1:l(1));
                c = c(l(1)+1:length(c));
        else
                app = c(1:prod(l(1,:)));
                c = c(prod(l(1,:))+1:length(c));
        end
        dimapp = length(app);
        nl2app = sum(app.^2);
        n0app = length(find(app==0));
else            % all coeffs can be thresholded.
        dimapp = 0; nl2app = 0; n0app = 0;
end

% Compute compression scores.
thresh = sort(abs(c));
onet = ones(size(thresh));
rl2scr = cumsum(thresh.^2) / (sum(thresh.^2)+nl2app);
rl2scr  = 100 *(onet - rl2scr);
n0scr = ((1:length(thresh))+(onet*n0app)) / (length(thresh)+dimapp);
n0scr = 100 * (n0scr);

thresh =[0 thresh];
rl2scr = [100 rl2scr];
n0scr = [0 n0scr];                              

% Find threshold for which the two scores are the same.
crit = abs(rl2scr-n0scr);
[xx,argmin] = min(crit);
suggthr = thresh(argmin);

% Set suggested threshold.
suggthr = thresh(argmin);
if dim==2, suggthr = sqrt(suggthr); end



