function thr = wdcbm(c,l,alpha,m)
%WDCBM Wavelet 1D denoising or compression using Birge-Massart strategy.
%       THR = WDCBM(C,L,ALPHA,M) returns level-dependent thresholds
%       for denoising or compression obtained by wavelet coefficients
%       selection rule using Birge-Massart strategy.
%
%       [C,L] is the wavelet decomposition structure of the signal
%       to be denoised or compressed.
%       ALPHA must be a real greater than 1. Typically ALPHA = 1.5 for 
%       compression and ALPHA = 3 for denoising. 
%       M is a positive integer, typically M = L(1)/2 the length of
%       coarsest approximation coefficients over 2.
%
%       With respect to Birge-Massart paper J0 becomes J0+1.    
%       J0 = length(L)-2 (and then [C,L] is the decomposition
%       of the signal at level J0). 
%       J0, M and ALPHA define the strategy:
%       - at level J0+1 (and coarser levels), everything is kept.
%       - for level j from 1 to J0, the nj larger coefficients
%       are kept with nj = M/(J0+1-j)^ALPHA. 
%       M is a tuning parameter (see default value).
%
%       WDCBM(C,L,ALPHA) is equivalent to WDCBM(C,L,ALPHA,L(1)/2). 

%       See also WDEN, WDENCMP, WPDENCMP.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:20:40 $

% Check arguments.
if errargn('wdcbm',nargin,[2:4],nargout,[0 1]), error('*'), end
if errargt('wdcbm',alpha-1,'rep'), error('*'), end
if nargin==4     
        if errargt('wdcbm',m,'int'), error('*'), end
else            
        m = l(1)/2;     % the number of detail coefs at level j0+1. 
end
j0 = length(l)-2;       % low frequency cutoff. 

% Wavelet coefficients selection.
for j=1:j0
        % number of coefs to be kept.
        n = m/(j0+1-j)^alpha;   
        n = min(round(n),l(j0-j+2));
        % threshold.
        if n == l(j0-j+2)
                thr(j) = 0;
        else
                d = detcoef(c,l,j);
                d = sort(abs(d));
                thr(j) = d(length(d)-n);
        end
end

