function ent = wentropy(x,t_ent,in3)
%WENTROPY Entropy(wavelet packet).
%       E = WENTROPY(X,T,P) returns the entropy E of the
%       vector or matrix X. 
%       In both cases, output E is a real number.
%       T is a string containing the type of entropy:
%               T = 'shannon', 'threshold', 'norm', 
%               'log energy', 'sure', 'user'.
%       P is an optional parameter depending on T value:
%               If T = 'shannon' or 'log energy', P is not used.
%               If T = 'threshold' or 'sure', P is the 
%               threshold and must a positive number.
%               If T = 'norm', P is the power and must be such 
%               that 1 <= P < 2.
%               If T = 'user', P is a string containing  
%               the M-file name of your own entropy
%               function, with a single input X.
%
%       E = WENTROPY(X,T) is equivalent to E = WENTROPY(X,T,0).

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:21:05 $

%       Uses ERRARGN, ERRARGT, WNOBLANK.

% Check arguments.
if errargn('wentropy',nargin,[2 3],nargout,[0 1]), error('*'); end

if nargin == 2
        in3 = 0.0;
else
        if isempty(in3)
                errargt('wentropy','invalid argument value','msg'); 
                error('*');  
        end
end
x = x(:);

% t_ent = 'shannon' and in3 not used
if strcmp(t_ent,'shannon')
        x = x(find(x)).^2;
        ent = -sum(x.*log(eps+x));

% t_ent = 'threshold' and in3 = epsilon
elseif (strcmp(t_ent,'threshold')) & (nargin == 3)
        if in3 < 0
                errargt('wentropy','invalid argument value','msg'); 
                error('*');  
        end
        x = abs(x);
        ent = sum(x > in3);

% t_ent = 'sure' and in3 = epsilon
elseif (strcmp(t_ent,'sure')) & (nargin == 3)
        if in3 < 0 
                errargt('wentropy','invalid argument value','msg'); 
                error('*');  
        end
        n = length(x);
        x2 = x.^2; t2 = in3.^2;
        xgt =  sum(x2 > t2); xlt = n - xgt;
        ent = n - (2*xlt) + (t2*xgt) + sum(x2.*(x2 <= t2));

% t_ent = 'norm' and in3 = p , lp norm ^ p
elseif (strcmp(t_ent,'norm')) & (nargin == 3)
        if in3 < 1 | in3 >= 2
                errargt('wentropy','invalid argument value','msg'); 
                error('*');  
        end
        x = abs(x);
        ent = sum(x.^in3);

% t_ent = 'energy'  and in3 not used  ( same as log energy)
elseif strcmp(t_ent,'energy')
        x = x(find(x)).^2;
        ent = sum(log(x));
          
% t_ent = 'log energy'  and in3 not used 
elseif strcmp(wnoblank(t_ent),'logenergy')
        x = x(find(x)).^2;
        ent = sum(log(x));
          
% t_ent = 'user' and in3 = '<function>' , user entropy
elseif (strcmp(t_ent,'user')) & (nargin == 3)
        ent = feval(in3,x);

else
        errargt('wentropy','invalid argument value','msg'); 
        error('*');  
end

prec = 1.0E-10;
if abs(ent)<prec
        ent = 0;
end
