function [x,xn] = wnoise(num,n,snrat,init)
%WNOISE Generate noisy wavelet test data.
%       X = WNOISE(NUM,N) returns values of test function 
%       number NUM, on a 2^N sample of [0,1].
%
%       [X,XN] = WNOISE(NUM,N,SNRAT) returns previous vector X
%       rescaled such that std(x) = SNRAT, 
%       and in vector XN the values of vector X corrupted by an
%       additive Gaussian white noise N(0,1), then XN has a
%       signal-to-noise ratio of SNRAT.
%
%       [X,XN] = WNOISE(NUM,N,SNRAT,INIT) returns previous
%       vector X and XN, but the generator seed is set to INIT
%       value. 
%
%       The six fonctions are due to Donoho, Johnstone in
%       "Adapting to unknown smoothness via wavelet shrinkage"
%       Preprint Stanford, january 93, p 27-28.
%       NUM = 1         Blocks
%       NUM = 2         Bumps
%       NUM = 3         Heavy sine
%       NUM = 4         Doppler
%       NUM = 5         Quadchirp
%       NUM = 6         Mishmash
%
%       See also WDEN.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:23:34 $

if nargin==2
if errargn('wnoise',nargin,[2],nargout,[0 1]), error('*'), end
else
if errargn('wnoise',nargin,[3:4],nargout,[0:2]), error('*'), end
end

t=[0.1 0.13 0.15 0.23 0.25 0.40 0.44 0.65 0.76 0.78 0.81];
h=[4   -5   3    -4   5    -4.2 2.1  4.3  -3.1 5.1  -4.2];     
if num==1       % blocks.
        tt=linspace(0,1,2^n);  x=zeros(1,length(tt));
        for j=1:11
                x = x + ( h(j)*(1+sign(tt-t(j)))/2 );
        end
elseif num==2   % bumps.
        h=abs(h);     
        w=0.01*[0.5 0.5 0.6 1 1 3 1 1 0.5 0.8 0.5];
        tt=linspace(0,1,2^n);  x=zeros(1,length(tt));
        for j=1:11
                x = x + ( h(j) ./ (1+ ((tt-t(j))/w(j)).^4));
        end
elseif num==3   % heavy sine.
        x=linspace(0,1,2^n);   
        x = 4*sin(4*pi*x) - sign(x-0.3) - sign(0.72-x);
elseif num==4   % doppler.
        x=linspace(0,1,2^n);   
        epsil = 0.05;
        x = sqrt(x.*(1-x)) .* sin( 2*pi*(1+epsil) ./ (x+epsil) );
elseif num==5   % quadchirp.
        t=linspace(0,1,2^n);   
        x = sin( (pi/3) * t .* (2^n * t.^2) );
elseif num==6   % mishmash.
        t=linspace(0,1,2^n);
        x = sin( (pi/3) * t .* (2^n * t.^2) );
        x = x + sin( pi * (2^n * 0.6902) * t );
        x = x + sin( pi * t .* (2^n * 0.125 * t) );
else
        errargt('wnoise','invalid argument value','msg');
        error('*')
end    
   
if nargin>=3 
        x = (x * (snrat/std(x)));
        if nargin==4, randn('seed',init); end
        xn = x + randn(1,2^n);
end
