function [thresh,rl2scr,n0scr,imin,sthr] = wpcmpscr(tree,data,in3)
%WPCMPSCR 1D or 2D Wavelet packets compression scores.
%       [THR,RL2,NZ,IMIN,STHR] = WPCMPSCR(TREE,DATA) returns
%       for the input wavelet packets decomposition structure 
%       [TREE,DATA], compression scores and suggested
%       threshold when approximation is kept:
%       THR the vector or ordered thresholds.
%       and the corresponding vectors of scores induced by
%       a THR(i)-thresholding :
%       RL2 vector of 2-norm recovery score in percent.
%       NZ  vector of relative number of zeros in percent.
%       IMIN is the index of THR for which the two scores are
%       approximately the same.
%       STHR is a suggested threshold.
%
%       When used with three arguments WPCMPSCR(TREE,DATA,IN3) 
%       returns the same outputs but all coefficients can be 
%       thresholded.
%
%       See also KEY2INFO, WCMPSCR.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:26:47 $

% Check arguments and set problem dimension.
if errargn('wpcmpscr',nargin,[2,3],nargout,[0:3 5]), error('*'), end
sizdat = wdatamgr('rsizes',data);
dim = size(sizdat,1);
keepapp = (nargin == 2);

% Set possible thresholds.
if keepapp      % approximation is kept.
        c = wdatamgr('rallcfs',data,tree);
        [ternod,sizes]  = wtreemgr('readall',tree);
        dimapp = prod(sizes(1,:));
        app = c(1:dimapp);
        c = c(dimapp+1:length(c));
        nl2app = sum(app.^2);
        n0app = length(find(app==0));
else            % all coeffs can be thresholded.
        dimapp = 0; nl2app = 0; n0app = 0;
end

% Compute compression scores.
thresh = sort(abs(c));
onet = ones(size(thresh));
rl2scr = cumsum(thresh.^2) / (sum(thresh.^2)+nl2app);
rl2scr  = 100 *(onet - rl2scr);
n0scr = ((1:length(thresh))+(onet*n0app)) / (length(thresh)+dimapp);
n0scr = 100 * (n0scr);

thresh =[0 thresh];
rl2scr = [100 rl2scr];
n0scr = [0 n0scr];    

% Find threshold for which the two scores are the same.
crit            = abs(rl2scr-n0scr);
[xx,imin]       = min(crit);
suggthr         = thresh(imin);

% Set suggested threshold.
if dim==2, 
        sthr = sqrt(suggthr);
else
        sthr = suggthr;
end



