function [tree,data] = wpdec(x,depth,wname,type_ent,parameter)
%WPDEC Wavelet packet decomposition 1-D.
%       [T,D] = WPDEC(X,N,'wname',E,P) returns a tree structure T
%       and a data structure D (see MAKETREE), corresponding to a
%       wavelet packet decomposition of the vector X, at level N,
%       with a particular wavelet ('wname', see WFILTERS).
%       E is a string containing the type of entropy (see WENTROPY):
%       E = 'shannon', 'threshold', 'norm', 'log energy', 'user'
%       P is an optional parameter:
%            'shannon' or 'log energy' : P is not used
%            'threshold' or 'sure'     : P is the threshold (0 <= P)
%            'norm' : P is a power (1 <= P < 2)
%            'user' : P is a string containing the name
%                     of an user-defined function.
%
%       [T,D] = WPDEC(X,N,'wname') is equivalent to
%       [T,D] = WPDEC(X,N,'wname','shannon').
%
%       See also MAKETREE, WAVEINFO, WDATAMGR, WENTROPY
%                WPDEC2, WTREEMGR.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:27:18 $

%       Uses MAKETREE, DWT, WENTROPY, WDATAMGR, WTREEMGR,
%            WFILTERS.

% Check arguments.
if errargn('wpdec',nargin,[3:5],nargout,[0:2]), error('*'); end
if      nargin==3
        type_ent = 'shannon';
        parameter = 0.0;
elseif  nargin==4 , parameter = 0.0; 
end

if strcmp(lower(type_ent),'user')
        if ~isstr(parameter)
                error('*** Invalid function name for user entropy ***');
        end
end

[mi,ind] = min(size(x));
if (ind==1) & (mi<2)
        x_shape = 'r';
elseif (ind==2) & (mi<2)
        x_shape = 'c';
else
        errargt('wpdec','invalid argument value','msg'); error('*');
end

% Initialization
%%%%%%%%%%%%%%%%
order       = 2;
[tree,nbtn] = maketree(order,depth,order/2);
sizes       = zeros(1,depth+1);
data        = x(:)';
ent         = zeros(1,(order*nbtn-1)/(order-1));
tmp         = NaN;
ent_opt     = tmp(ones(size(ent)));

lx          = length(x);
sizes(1)    = lx;
ent(1)      = wentropy(x,type_ent,parameter);

% Tree computation
%%%%%%%%%%%%%%%%%%
[LoF_D,HiF_D] = wfilters(wname,'d');
n = 2;
for k=0:depth-1
        beg = 1;
        for p=0:order^k-1
                x        = data(beg:beg+lx-1);
                [a,d]    = dwt(x,LoF_D,HiF_D);
                data     = [data(1:beg-1) a d data(beg+lx:length(data))];
                ent(n)   = wentropy(a,type_ent,parameter);
                ent(n+1) = wentropy(d,type_ent,parameter);
                beg      = beg+2*length(a);
                n        = n+order;
        end
        lx         = length(a);
        sizes(k+2) = lx;
end
tree = wtreemgr('winfo',tree,lx*ones(1,nbtn));

% Writing wavelet and entropy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data = wdatamgr('write',data,sizes,x_shape,...
                ent,ent_opt,parameter,type_ent,wname,order);
