function [tree,data] = wpdec2(x,depth,wname,type_ent,parameter)
%WPDEC2 Wavelet packet decomposition 2-D.
%       [T,D] = WPDEC2(X,N,'wname',E,P) returns a tree structure T
%       and a data structure D (see MAKETREE), corresponding to a
%       wavelet packet decomposition of the matrix X, at level N,
%       with a particular wavelet ('wname', see WFILTERS).
%       E is a string containing the type of entropy
%       (see WENTROPY):
%       E = 'shannon', 'threshold', 'norm', 'log energy', 'sure',
%           'user'
%       P is an optional parameter:
%            'shannon' or 'log energy' : P is not used
%            'threshold' or 'sure'     : P is the threshold (0 <= P)
%            'norm' : P is a power (1 <= P < 2)
%            'user' : P is a string containing the name
%                     of an user-defined function.
%
%       [T,D] = WPDEC2(X,N,'wname') is equivalent to
%       [T,D] = WPDEC2(X,N,'wname','shannon').
%
%       See also MAKETREE, WAVEINFO, WDATAMGR,  WENTROPY
%                WPDEC, WTREEMGR.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:27:29 $

%       Uses DWT2, MAKETREE, WDATAMGR, WENTROPY, WFILTERS,
%            WTREEMGR.

% Check arguments.
if errargn('wpdec2',nargin,[3:5],nargout,[0:2]), error('*'); end
if      nargin==3
        type_ent  = 'shannon';
        parameter = 0.0;
elseif  nargin==4 , parameter = 0.0; 
end
if strcmp(lower(type_ent),'user')
        if ~isstr(parameter)
                error('*** Invalid function name for user entropy ***');
        end
end

% Initialization
%%%%%%%%%%%%%%%%
x_shape = 'm';
order        = 4;
[tree,nbtn] = maketree(order,depth,order/2);
sizes       = zeros(2,depth+1);
data        = x(:)';
ent         = zeros(1,(order*nbtn-1)/(order-1));
tmp         = NaN;
ent_opt     = tmp(ones(size(ent)));

sizes(:,1)  = size(x)';
sx          = prod(size(x));
ent(1)      = wentropy(x,type_ent,parameter);

% Tree Computation
%%%%%%%%%%%%%%%%%%
[LoF_D,HiF_D] = wfilters(wname,'d');
n = 2;
for k=0:depth-1
        beg = 1;
        s = sizes(:,k+1);
        for p=0:order^k-1
                x = reshape(data(beg:beg+sx-1),s(1),s(2));
                [a,h,v,d] = dwt2(x,LoF_D,HiF_D);
                data = [data(1:beg-1) a(:)' h(:)' v(:)' d(:)' ...
                        data(beg+sx:length(data))];
                ent(n)   = wentropy(a,type_ent,parameter);
                ent(n+1) = wentropy(h,type_ent,parameter);
                ent(n+2) = wentropy(v,type_ent,parameter);
                ent(n+3) = wentropy(d,type_ent,parameter);
                beg      = beg+order*prod(size(a));
                n        = n+order;
        end
        sx = prod(size(a));
        sizes(:,k+2) = size(a)';
end
tree = wtreemgr('winfo',tree,size(a)'*ones(1,nbtn));

% Writing wavelet and entropy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
data = wdatamgr('write',data,sizes,x_shape,...
                ent,ent_opt,parameter,type_ent,wname,order);
