function [xd,treed,datad,perf0,perfl2] = ...
         wpdencmp(in1,in2,in3,in4,in5,in6,in7,in8)
%WPDENCMP De-noising or compression using wavelet packet.
%       [XD,TREED,DATAD,PERF0,PERFL2] =
%       WPDENCMP(X,SORH,N,'wname',CRIT,PAR,KEEPAPP)
%       returns a de-noised or compressed version XD of input
%       signal X (1-D or 2-D) obtained by wavelet packet
%       coefficients thresholding.
%       Additional output arguments [TREED,DATAD] are the
%       wavelet packet best decomposition structure of XD,
%       PERFL2 and PERF0 are L^2 recovery and compression
%       scores in percentages.
%       PERFL2 = 100*(vector-norm of WP-cfs of XD)^2 over
%       (vector-norm of WP-cfs of X)^2
%       SORH ('s' or 'h') is for soft or hard thresholding
%       (see WTHRESH for more details).
%       Wavelet packet decomposition is performed at level N
%       and 'wname' is a string containing wavelet name.
%       Best decomposition is performed using entropy criterion
%       defined by string CRIT and parameter PAR (see WENTROPY
%       for details). Threshold parameter is also PAR.
%       If KEEPAPP = 1, approximation coefficients cannot be
%       thresholded, otherwise it is possible.
%
%       [XD,TREED,DATAD,PERF0,PERFL2] =
%       WPDENCMP(TREE,DATA,SORH,CRIT,PAR,KEEPAPP)
%       has same output arguments, using the same options as
%       above, but obtained directly from the input wavelet
%       packet decomposition structure [TREE,DATA] of the
%       signal to be de-noised or compressed.
%       In addition if CRIT = 'nobest' no optimization is done
%       and the current decomposition is thresholded.
%
%       See also DDENCMP, WDENCMP, WENTROPY, WPDEC, WPDEC2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:27:36 $

%       Uses BESTTREE, ENTRUPD, WDATAMGR, WPDEC, WPDEC2, WPRCOEF,
%            WPTHCOEF.

% Check arguments and set problem dimension.
if nargin==6
        if errargn('wpdencmp',nargin,[6],nargout,[0 1 3:5])
                error('*') 
        end
        tree=in1; data=in2; 
        sorh=in3; crit=in4; par=in5; keepapp=in6;
        sizdat = wdatamgr('rsizes',data);
        dim    = size(sizdat,1);
else
        if errargn('wpdencmp',nargin,[7],nargout,[0 1 3:5])
                error('*')
        end
        x=in1; 
        sorh=in2; n=in3; w=in4; crit=in5; par=in6; keepapp=in7;
        if errargt('wpdencmp',n,'int'), error('*'), end
        if errargt('wpdencmp',w,'str'), error('*'), end

        dim = 1; if min(size(x)) ~= 1, dim = 2; end
end
if errargt('wpdencmp',sorh,'str'), error('*'), end
if errargt('wpdencmp',crit,'str'), error('*'), end

if nargin==7
        % Wavelet packet decomposition of x.
        if dim==1
                [tree,data] = wpdec(x,n,w,crit,par);
        else
                [tree,data] = wpdec2(x,n,w,crit,par);
        end
elseif strcmp(crit,'nobest') == 0
        % Update entropy.
        data = entrupd(tree,data,crit,par);
end

if strcmp(crit,'nobest') == 0
        % Perform best tree.
        [tree,data] = besttree(tree,data);
end

% Wavelet packet coefficients thresholding.
datad = wpthcoef(data,tree,keepapp,sorh,par);
treed = tree;

% Wavelet packet reconstruction of xd.
xd = wpcoef(treed,datad,0);

if nargout<4 , return; end

% Compute L^2 recovery and compression scores.
% Extract final coefficients after thresholding.
cfs = wdatamgr('rallcfs',datad);

% Compute compression score.
perf0 = 100*(length(find(cfs==0))/length(cfs));
if nargout<5 , return; end

% Extract final coefficients before thresholding.
orcfs = wdatamgr('rallcfs',data);

% Compute L^2 recovery score.
perfl2 = 100*(sum(cfs.^2)/sum(orcfs.^2));
