function [tree,data,x] = wpjoin(tree,data,node)
%WPJOIN Recompose wavelet packet.
%       WPJOIN updates the tree and data structures after the
%       recomposition of a node.
%
%       [T,D] = WPJOIN(T,D,N) returns the modified tree
%       structure T and the modified data structure D, (see
%       MAKETREE) corresponding to a recomposition of the node N.
%       [T,D,X] = WPJOIN(T,D,N) also returns the coefficients
%       of the node.
%
%       [T,D] = WPJOIN(T,D)   is equivalent to
%               [T,D] = WPJOIN(T,D,0).
%       [T,D,X] = WPJOIN(T,D) is equivalent to
%               [T,D,X] = WPJOIN(T,D,0).
%
%       See also MAKETREE, WPDEC, WPDEC2, WPSPLT.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:28:25 $

%       Uses DEPO2IND, IDWT, IDWT2, IND2DEPO, WDATAMGR, WFILTERS.  

% Check arguments.
if errargn('wpjoin',nargin,[2:3],nargout,[1:3]), error('*'); end
if nargin == 2, node = 0; end

order      = wtreemgr('order',tree);
depth      = wtreemgr('depth',tree);
node       = depo2ind(order,node);
pack_exist = wtreemgr('istnode',tree,node);

% For a terminal node
%%%%%%%%%%%%%%%%%%%%%
if pack_exist ~= 0
        x = wdatamgr('rcfs',data,tree,pack_exist);
        return
end

tab        = wtreemgr('table',tree,0);
[row,col]  = find(tab==node);
icol = find(col>2);

rec        = tab(row(icol),2:max(col(icol)));
rec        = rec(:);
rec        = rec(find(rec>node));

if length(rec>1);
        rec = sort(rec);
        K   = [1;find(diff(rec)~=0)+1];
        rec = rec(K);
end
rec    = wrev([node;rec])';
ord    = [1:order];
i_fils = order*(rec(ones(1,order),:))'+ ord(ones(1,length(rec)),:);
for k = 1:length(rec)
        for j = 1:order
                [r,c] = find(tab==i_fils(k,j));
                i_fils(k,j)  = min(r);        % indices
        end
end

i_size = ind2depo(order,rec')+1;

% Recomposition
%%%%%%%%%%%%%%%
filter = wdatamgr('read_wave',data);
[LoF_R,HiF_R] = wfilters(filter,'r');
sizes  = wdatamgr('rsizes',data);

for k = 1:length(rec)
        left_child = i_fils(k,1);
        if order == 2
                % [a,d,beg,len] = wdatamgr('rmcfs',data,tree,left_child);
                %--------------------------------------------------------
                beg = sum(tree(2,1:left_child-1))+1;
                len = tree(2,left_child);
                beg = beg+3;        % shift for dim & type of structure
                a   = data(beg:beg+len-1);
                d   = data(beg+len:beg+len+len-1);
                sx  = sizes(i_size(k));
                x   = idwt(a,d,LoF_R,HiF_R,sx);
        else
                % [a,h,v,d,beg,len] = wdatamgr('rmcfs',data,tree,left_child);
                %-------------------------------------------------------------
                % attention prod([])=1
                if left_child ~= 1
                        beg = sum(prod(tree(2:3,1:left_child-1)))+1;
                else
                        beg = 1;
                end
                len = prod(tree(2:3,left_child));
                beg = beg+3;        % shift for dim & type of structure
                r   = tree(2,left_child);
                c   = tree(3,left_child);
                lim = beg:len:beg+order*len;
                a   = reshape(data(lim(1):lim(2)-1),r,c);
                h   = reshape(data(lim(2):lim(3)-1),r,c);
                v   = reshape(data(lim(3):lim(4)-1),r,c);
                d   = reshape(data(lim(4):lim(5)-1),r,c);
                sx  = sizes(:,i_size(k));
                x   = idwt2(a,h,v,d,LoF_R,HiF_R,sx);
        end
        % data = wdatamgr('replace',data,x(:)',beg,beg+order*len-1);
        %-----------------------------------------------------------
        first       = beg;
        last        = beg+order*len-1;
        data        = [data(1:first-1) x(:)' data(last+1:length(data))];
        data(2)     = data(2)-order*len+prod(sx);

        % insert_st = [rec(k) zeros(1,order-1);sx zeros(order/2,order-1)];
        % tree = wtreemgr('change',tree,insert_st,i_fils(k,:));
        %-----------------------------------------------------------------
        tree(:,i_fils(k,:)) = [rec(k) zeros(1,order-1);sx zeros(order/2,order-1)];
end

%
% Modification of Tree and Data Structures.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tree = wtreemgr('restore',tree);
new_depth = wtreemgr('depth',tree);
if new_depth < depth
        nbs   = size(sizes,2)-(depth-new_depth);
        data  = wdatamgr('wsizes',data,sizes(:,1:nbs));
        nbn   = (order^(new_depth+1)-1)/(order-1);
        ent   = wdatamgr('read_ent',data,[0:nbn-1]');
        data  = wdatamgr('write_ent',data,ent);
        ent   = wdatamgr('read_ento',data,[0:nbn-1]');
        data  = wdatamgr('write_ento',data,ent);
end
