function x = wrcoef(o,c,l,arg4,arg5,arg6)
%WRCOEF Reconstruct single branch from 1-D wavelet coefficients.
%       WRCOEF reconstructs the coefficients of a 1-D signal,
%       given a wavelet decomposition structure (C and L) and
%       either a specified wavelet ('wname', see WFILTERS) or 
%       specified reconstruction filters (Lo_R and Hi_R).
%
%       X = WRCOEF('type',C,L,'wname',N) computes the vector of
%       reconstructed coefficients, based on the wavelet
%       decomposition structure [C,L] (see WAVEDEC), at level N.
%       'wname' is a string containing the name of wavelet.
%       Argument 'type' determines whether approximation
%       ('type' = 'a') or detail ('type' = 'd') coefficients are
%       reconstructed.
%       When 'type' = 'a', N is allowed to be 0, otherwise 
%       strictly positive N is required.
%       Level N must be an integer such that N <= length(L)-2. 
%
%       X = WRCOEF('type',C,L,Lo_R,Hi_R,N) computes coefficient
%       as above, given the reconstruction you specify.
%
%       X = WRCOEF('type',C,L,'wname') and
%       X = WRCOEF('type',C,L,Lo_R,Hi_R) reconstruct coefficients
%       of maximum level N = length(L)-2.
%
%       See also APPCOEF, DETCOEF, WAVEDEC.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:30:06 $

%       Uses APPCOEF, DETCOEF, DYADUP, ERRARGN, ERRARGT, WFILTERS,
%            WKEEP.

% Check arguments.
if errargn('wrcoef',nargin,[4:6],nargout,[0:1]), error('*'), end
o = lower(o(1));
rmax = length(l);       nmax = rmax-2;
if o=='a', nmin = 0; else , nmin = 1; end
if isstr(arg4)
        [LoF_R,HiF_R] = wfilters(arg4,'r');
        if nargin == 5, n = arg5; else, n = nmax; end
else
        LoF_R = arg4;   HiF_R = arg5;
        if nargin == 6, n = arg6; else, n = nmax; end
end
if (n<nmin) | (n>nmax) | (n~=fix(n))
        errargt('wrcoef','invalid level value','msg'); error('*');
end

s = l(rmax);
if o=='a'
        % Extract approximation.
        a = appcoef(c,l,LoF_R,HiF_R,n);

        % Approximation reconstruction.
        for k=1:n
                a = conv(dyadup(a),LoF_R);
        end
        x = wkeep(a,s);
elseif o=='d'
        % Extract detail coefficients.
        d = detcoef(c,l,n);

        % Detail reconstruction.
        d = conv(dyadup(d),HiF_R);
        for k=2:n
                d = conv(dyadup(d),LoF_R);
        end
        x = wkeep(d,s);
else
        errargt('wrcoef','invalid argument value','msg'); error('*');
end
