function x = wrcoef2(o,c,s,arg4,arg5,arg6)
%WRCOEF2 Reconstruct single branch from 2-D wavelet coefficients.
%       WRCOEF2 reconstructs the coefficients of an image.
%
%       X = WRCOEF2('type',C,S,'wname',N) computes the matrix of
%       reconstructed coefficients of level N, based on the 
%       wavelet decomposition structure [C,S] (see WAVEDEC2).
%       'wname' is a string containing the name of the wavelet.
%       If 'type' = 'a', approximation coefficients are reconstructed
%       otherwise if 'type' = 'h' ('v' or 'd' respectively), 
%       horizontal (vert. or diag. respectively) detail coefficients
%       are reconstructed.
%
%       Level N must be an integer such that:
%       0 <= N <= size(S,1)-2 if 'type' = 'a' and such that 
%       1 <= N <= size(S,1)-2 if 'type' = 'h', 'v'or 'd'.
%
%       Instead of giving the wavelet name, you can give the filters.
%       For X = WRCOEF2('type',C,S,Lo_R,Hi_R,N),  
%       Lo_R is the reconstruction low-pass filter and
%       Hi_R is the reconstruction high-pass filter
%
%       X = WRCOEF2('type',C,S,'wname') or
%       X = WRCOEF2('type',C,S,Lo_R,Hi_R) reconstructs 
%       coefficients of maximum level N = size(S,1)-2.
%
%       See also APPCOEF2, DETCOEF2, WAVEDEC2.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:30:12 $

%       Uses APPCOEF2, DETCOEF2, DYADUP, ERRARGN, ERRARGT,
%            WFILTERS, WKEEP.


% Check arguments.
if errargn('wrcoef2',nargin,[5 6],nargout,[0 1]), error('*'), end
rmax = size(s,1);       nmax = rmax-2;
o    = lower(o(1));
if o=='a', nmin = 0; else, nmin = 1; end

if isstr(arg4)
        [LoF_R,HiF_R] = wfilters(arg4,'r');
        if nargin == 5, n = arg5; else, n = nmax; end
else
        LoF_R = arg4;   HiF_R = arg5;
        if nargin == 6, n = arg6; else, n = nmax; end
end
if find((n<nmin) | (n>nmax) | (n~=fix(n)))
        errargt('wrcoef2','invalid level value','msg'); error('*');
end

if o=='a'
        x = appcoef2(c,s,LoF_R,HiF_R,n);
        if n==0, return; end
else
        x = detcoef2(o,c,s,n);
end

if     strcmp(o,'a'),   f1 = LoF_R;     f2 = LoF_R;
elseif strcmp(o,'h'),   f1 = HiF_R;     f2 = LoF_R;
elseif strcmp(o,'v'),   f1 = LoF_R;     f2 = HiF_R;
elseif strcmp(o,'d'),   f1 = HiF_R;     f2 = HiF_R;
else,  errargt('wrcoef2','invalid argument value','msg'); error('*');
end

s = s(rmax-n+1:rmax,:);
x = conv2(dyadup(x,'r'),f1');
x = conv2(dyadup(x,'c'),f2);

for p = 2:n
        x = conv2(dyadup(x,'r'),LoF_R');
        x = conv2(dyadup(x,'c'),LoF_R);
end
x = wkeep(x,s(n,:));
