function m = wrmcoef(o,c,l,arg4,arg5,arg6)
%WRMCOEF Reconstruct row matrix of single branches 
%         from 1D wavelet coefficients.
%       M = WRMCOEF(O,C,L,W,N) computes the matrix of
%       reconstructed coefficients, based on the wavelet
%       decomposition structure [C,L], of levels given 
%       in vector N.
%       W is a string containing the wavelet name.
%       If O = 'a', approximation coefficients are reconstructed
%       and value 0 for level is allowed, else detail coefficients 
%       are reconstructed and only strictly positive values for
%       level are allowed.
%       Vector N must contains positive integers <= length(L)-2. 
%
%       M is the output matrix of reconstructed coefficients 
%       vectors stored rowise.
%       
%       For M = WRMCOEF(O,C,L,Lo,Hi,N) 
%       Lo is the reconstruction low-pass filter and
%       Hi is the reconstruction high-pass filter.
%
%       M = WRMCOEF(O,C,L,W) or M = WRMCOEF(O,C,L,Lo,Hi) reconstructs 
%       coefficients of all possible levels.
%
%       See also APPCOEF, DETCOEF, WRCOEF, WAVEDEC.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:30:31 $

%       Uses DETCOEF, WKEEP, UPWLEV, DYADUP, WFILTERS.

% Check arguments.
if errargn('wrmcoef',nargin,[4:6],nargout,[0:1]), error('*'), end
o = lower(o(1));
rmax = length(l);       nmax = rmax-2;
if o=='a', nmin = 0; else , nmin = 1; end
if isstr(arg4)
        [LoF_R,HiF_R] = wfilters(arg4,'r');
        if nargin == 5, n = arg5; else, n = [nmin:nmax]; end
else
        LoF_R = arg4;   HiF_R = arg5;
        if nargin == 6, n = arg6; else, n = [nmin:nmax]; end
end
if find((n<nmin) | (n>nmax) | (n~=fix(n)))
        errargt('wrmcoef','invalid level(s) value(s)','msg'); error('*');
end

% Initialization
if size(l,1)>1 , c = c'; l = l'; end
s = l(rmax);
m = zeros(length(n),s);

if o=='a'
        for p = nmax:-1:0
                [c,l,a] = upwlev(c,l,LoF_R,HiF_R);
                j = find(p==n);
                if j
                        % Approximation reconstruction.
                        for k=1:p
                                a = conv(dyadup(a),LoF_R);
                        end
                        a = wkeep(a,s);
                        a = a(ones(size(j)),:);
                        m(j,:) = a;
                end
        end
elseif o=='d'
        for p = 1:nmax
                j = find(p==n);
                if j
                        % Extract detail coefficients.
                        d = detcoef(c,l,p);

                        %Detail reconstruction.
                        d = conv(dyadup(d),HiF_R);
                        for k=2:p
                                d = conv(dyadup(d),LoF_R);
                        end
                        d = wkeep(d,s);
                        d = d(ones(size(j)),:);
                        m(j,:) = d;
                end
        end
else
        errargt('wrmcoef','invalid argument value','msg'); error('*');
end

