function [out1,out2,out3,out4] = wtreemgr(opt,tree,in3,in4,in5)
%WTREEMGR Manager for tree structure.
%       [OUT1,OUT2,OUT3,OUT4] = WTREEMGR(OPT,STRUCTURE,IN3,IN4,IN5)
%       Allowed values for OPT and associated uses are 
%       described in the functions listed in the See Also section:        
%
%       'allnodes'  : All nodes.
%       'isnode'    : Is node.
%       'istnode'   : Lop "is terminal node".
%       'create'    : Create a tree.
%       'nodeasc'   : Node ascendants.
%       'nodedesc'  : Node descendants.
%       'nodepar'   : Node parent.
%       'ntnode'    : Number of terminal nodes.
%       'tnodes'    : Terminal nodes.
%       'order'     : Order of tree.
%       'depth'     : Depth of tree.
%
%       For tree structure implementation see MAKETREE.
%
%       See also ALLNODES, ISNODE, ISTNODE, MAKETREE, 
%                NODEASC, NODEDESC, NODEPAR, NTNODE, TNODES,
%                TREEDPTH, TREEORD.

%       M. Misiti, Y. Misiti, G. Oppenheim, J.M. Poggi 12-Mar-96.
%       Copyright (c) 1995-96 by The Mathworks, Inc.
%       $Revision: 1.1 $  $Date: 1996/03/05 21:31:48 $

%       Uses DEPO2IND, IND2DEPO, WTREEMGR, WCOMMON.

%Internal Options
%----------------
%       'replace'   : Replace.
%       'change'    : Change.
%       'restore'   : Restore.
%       'rinfo'     : Read Info.
%       'winfo'     : Write Info.
%       'table'     : table of nodes fathers.
%       'readall'   :         
%               out1 = Terminal Nodes (left ---> right)
%               out2 = sizes of Terminal Nodes
%               out3 = order
%               out4 = depth

% Check arguments.
if errargn('wtreemgr',nargin,[2:5],nargout,[0:4]), error('*'); end

if strcmp(opt,'create')
        % [in3 in4 in5] = [order depth nbinfo]
        % [out1 out2]   = [tree ntnode]
        out2  = in3^in4;
        r     = in5+1;
        c     = out2+1;
        out1  = zeros(r,c);
        out1(1,1:out2) = (out2-1)/(in3-1):(in3*out2-in3)/(in3-1);
        out1(1,c)      = -in3;
        return
end

[rows,cols] = size(tree);
ntnode      = cols-1;
if ntnode<1
        errargt('wtreemgr','Invalid tree structure!','msg'); error('*'); 
end

if strcmp(opt,'ntnode')
        out1 = ntnode;
        return
elseif strcmp(opt,'replace')
        % in3 = insert  in4,in5 = indices of replaced first, last
        if nargin==3
                out1 = in3;
        else
                out1 = [tree(:,1:in4-1) in3 tree(:,in5+1:cols)];
        end
        return
elseif strcmp(opt,'change')
        % in3 = new values      in4 = indices of changed
        if nargin==3
                out1 = in3;
        else
                out1 = tree;
                out1(:,in4) = in3;
        end     
        return
elseif strcmp(opt,'restore')
        % on trouve sur les tailles (ligne 2) 
        % et pas sur les indices de noeuds (ligne 1)
        %
        preserve = [find(tree(2,:)) cols];
        out1 = tree(:,preserve);
        return
elseif strcmp(opt,'rinfo')
        if rows>1
                out1 = tree(2:rows,1:ntnode);
        else
                out1 = [];
        end
        return
elseif strcmp(opt,'winfo')
        out1  = tree;
        if rows>1
                out1(2:rows,1:ntnode) = in3;
        end
        return
end

order = abs(tree(1,cols));
depth = floor(log(max(tree(1,:))*(order-1)+1)/log(order));

if strcmp(opt,'order')
        out1 = order;

elseif strcmp(opt,'depth')
        out1 = depth;

elseif strcmp(opt,'readall')
        out1 = tree(1,1:ntnode)';
        out2 = tree(2:rows,1:ntnode)';
        out3 = order;
        out4 = depth;        

elseif strcmp(opt,'tnodes')
        [out1,out2] = sort(tree(1,1:ntnode)');
        [nul,out2]  = sort(out2);
        if nargin==3
                [out1(:,1),out1(:,2)] = ind2depo(order,out1);
        end

elseif strcmp(opt,'allnodes')
        out1  = wtreemgr('tnodes',tree);
        if length(out1)<=1, return; end
        tmp   = out1;
        for k = 1:depth
                tmp  = floor((tmp-1)/order);
                out1 = [out1;tmp];
        end
        out1 = sort(out1(find(out1>=0)));
        K    = [1;find(diff(out1)~=0)+1];
        out1 = out1(K);
        if nargin==3
                [out1(:,1),out1(:,2)] = ind2depo(order,out1);
        end

elseif strcmp(opt,'isnode')
        node  = depo2ind(order,in3);
        all   = wtreemgr('allnodes',tree);
        if prod(size(node))<=1
                if find(all==node)
                        out1 = 1;
                else
                        out1 = 0;
                end
        else
                [out1,nul] = wcommon(node,all);
        end

elseif strcmp(opt,'istnode')
        node  = depo2ind(order,in3);
        tn    = tree(1,1:ntnode)';
        out1  = zeros(size(node));
        for k = 1:length(node)
                in = find(node(k)==tn);
                if ~isempty(in), out1(k) = in; end
        end

elseif strcmp(opt,'nodepar')
        out1  = floor((depo2ind(order,in3)-1)/order);
        if nargin==4
                [out1(:,1),out1(:,2)] = ind2depo(order,out1);
        end

elseif strcmp(opt,'nodeasc')
        out1  = depo2ind(order,in3);
        [d,p] = ind2depo(order,out1);
        tmp   = out1;
        for k = 1:d
                tmp  = floor((tmp-1)/order);
                out1 = [out1;tmp];
        end
        if nargin==4
                [out1(:,1),out1(:,2)] = ind2depo(order,out1);
        end

elseif strcmp(opt,'nodedesc')
        A     = wtreemgr('table',tree);
        node  = depo2ind(order,in3);
        [i,j] = find(A==node);
        out1  = A(i,:);
        out1  = out1(:);
        out1  = sort(out1(find(out1>=node)));
        if length(out1)>1
                K = [1;find(diff(out1)~=0)+1];
        else
                K = 1;
        end
        out1 = out1(K);
        if nargin==4
                [out1(:,1),out1(:,2)] = ind2depo(order,out1);
        end

elseif strcmp(opt,'table')
        [tn,K] = wtreemgr('tnodes',tree);
        if nargin==3, tn = tn(K); end
        out1 = zeros(length(tn),depth+1);
        out1(:,1) = tn;
        for j = 1:depth
                out1(:,j+1) = floor((out1(:,j)-1)/order);
        end

else
        errargt('wtreemgr','unknown option','msg'); error('*'); 
end
